from enum import IntEnum, StrEnum


class SecurityType(IntEnum):
    """ 回测品种 """
    CB = 1 # 可转债
    STK = 2 # 股票
    ETF = 3 # ETF
    FINANCE = 10 # 财报


class BenchmarkType(StrEnum):
    """ 基准类型 """
    STOCK = 'STOCK' # 正股
    EQUALWEIGHT = 'EQUALWEIGHT' # 等权
    HS300 = '1B0300.SH' # 沪深300
    ZZ1000 = '1B0852.SH' # 中证1000
    GZ2000 = '399303.SZ' # 国证2000
