SCREEN_COMMON_SETTING_RELATIVE_FACTORS: dict[str, list[str]] = {
    'exclude_area': ['area'],
    'exclude_first_industries': ['industry_1'],
    'exclude_second_industries': ['industry_2'],
    'exclude_third_industries': ['industry_3'],
    'exclude_orgform': ['orgform'],
    'exclude_org_types': ['orgform'],
}


SCREEN_CB_SETTING_RELATIVE_FACTORS: dict[str, list[str]] = {
    'exclude_st': ['name_stk'],
    'exclude_list_days': ['list_days'],
    'exclude_rating': ['rating'],
    'exclude_yy_rating': ['yy_rating'],
    'exclude_call': ['is_call'],
    'exclude_redeem_remain_days': ['redeem_remain_days'],
}


SCREEN_STK_SETTING_RELATIVE_FACTORS: dict[str, list[str]] = {
    'exclude_st': ['name'],
    'exclude_list_days': ['list_days'],
}


SCREEN_ETF_SETTING_RELATIVE_FACTORS: dict[str, list[str]] = {
}
