from typing import TypedDict

from .common import TaskBaseParameters


class GenBasketDataItem(TypedDict):
    """ 生成篮子参数数据项 """
    code: str # 编码
    weight: float # 权重
    price: float # 价格
    

class GenBasketParameters(TaskBaseParameters):
    """ 生成篮子参数 """
    capital: float # 初始资金
    board_lot: int # 每手数量
    basket_data: list[GenBasketDataItem] # 篮子数据


class GenBasketResultItem(TypedDict):
    """ 生成篮子结果数据项 """
    code: str # 编码
    target_num: int # 目标数量
