from enum import StrEnum


class SyncStatus(StrEnum):
    """ 同步结果 """
    SUCCESS = "success"
    NO_CHANGE = "no_change"
    FAILED = "failed"
    

class LoadTrigger(StrEnum):
    """ 加载触发器 """
    INTERVAL = "interval"
    CONTINUOUS = "continuous"
